/* eslint-disable */

/**
* @license Apache-2.0
*
* Copyright (c) 2018 The Stdlib Authors.
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*    http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/

(function() {
	'use strict';

	var idx;
	var str;
	var el;

	el = document.querySelector( 'h1' );

	str = [ 'Beep', 'Boop' ];
	idx = 0;

	setInterval( update, 1000 );

	function update() {
		idx = (idx+1) % 2;
		el.innerHTML = str[ idx ];
	}
})();
